/*==============================================================================*/
/*  Copyright (C) 2021, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   I2C Driver                                                       */
/*  AppVer :   DT+ Ver1.10`                                                    */
/*  FileID :   17C2                                                             */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Please customize the code for your environment.                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for Port Control                                             */
/*==============================================================================*/
#include "DTTestPointCountDefine.h"
#include "Common.h"
// #include <itron.h>
// #include <kernel.h>

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT+ Project setting.     */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Macro:  DT_INLINE                                                           */
/*  Desc:   Please use "static" instead of "inline" if "inline" cannot be used. */
/*==============================================================================*/
#define	DT_INLINE	inline
/* #define	DT_INLINE	static */

/*==============================================================================*/
/*  Func:   _TP_BusPortInit                                                     */
/*  Desc:   Please describe the code to initializes ports.                      */
/*==============================================================================*/
static void _TP_BusPortInit(void)
{
	/* ex)	
			PA.DIR = 0x3f;
			PD.DIR = 0x3f;
			PA.word = 0x3f;
			PD.word = 0x3f;	
	*/
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOA );
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOD );
	GPIOPinTypeGPIOOutput( GPIO_PORTD_BASE, GPIO_PIN_2 );		/*	CLK	*/
	GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_2 );		/*	DAT0	*/
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, GPIO_PIN_2);
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_2, GPIO_PIN_2 );
}

/*==============================================================================*/
/*  Func:   portSetCLK                                                          */
/*  Desc:   Please describe the code to set the value to CLK Port.              */
/*==============================================================================*/
DT_INLINE void portSetCLK( DT_UINT dat)
{
	/* ex)
		PA.bit.b0 = dat?1:0;
	*/
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, dat?0xff:0x00 );
}

/*==============================================================================*/
/*  Func:   portSetDATA                                                         */
/*  Desc:   Please describe the code to set the value to DATA Port              */
/*==============================================================================*/
DT_INLINE void portSetDATA(DT_UINT dat)
{
	/* ex)
	 PA.bit.b2 = dat?1:0;
	*/
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_2, dat?0xff:0x00 );
}

/*==============================================================================*/
/*  Func:   enterCritical                                                       */
/*  Desc:   Please describe the code to enter Critical Section.                 */
/*==============================================================================*/
DT_INLINE void enterCritical(void)
{
	/* ex)
		DI();
	*/ 
	taskENTER_CRITICAL();
}

/*==============================================================================*/
/*  Func:   exitCritical                                                        */
/*  Desc:   Please describe the code to exit Critical Section.                  */
/*==============================================================================*/
DT_INLINE void exitCritical(void)
{
	/* ex)
		EI();
	*/ 
	taskEXIT_CRITICAL();
}

/*==============================================================================*/
/*  Don't change the code from here as possible.                                */
/*==============================================================================*/
#define MAX_LOG_BIT_SIZE	((__DTTESTPoint_TotalCount + 7) / 8)
static unsigned char logBitBuf[MAX_LOG_BIT_SIZE];
static unsigned int bufPos = 0;

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define		DT_VARIABLE_BIT			0x02
#define		DT_TARGET_TIME_BIT		0x04
#define		DT_EVTTRG_BIT			0x08
#define		DT_VARIABLE_FAST_BIT	0x01
#define		DT_EVTTRG_FAST_BIT		0x02


/*==============================================================================*/
/*  Func:   portInit                                                            */
/*  Desc:   Initialize Port Function                                            */
/*==============================================================================*/
static int init = 0;
DT_INLINE void portInit(void)
{
	if( init == 0 ){
		_TP_BusPortInit();
		init = 1;
	}
}

/*==============================================================================*/
/*  Func:   _TP_BusOutStartCondition                                            */
/*  Desc:   Output Start Condition                                              */
/*==============================================================================*/
DT_INLINE void _TP_BusOutStartCondition(void)
{
	portSetDATA(0);
}

/*==============================================================================*/
/*  Func:   _TP_BusOutStopCondition                                             */
/*  Desc:   Output Stop Condition                                               */
/*==============================================================================*/
DT_INLINE void _TP_BusOutStopCondition(void)
{
	portSetDATA(1);
}

/*==============================================================================*/
/*  Func:   _TP_BusOutRestartCondition                                          */
/*  Desc:   Output Restart Condition                                            */
/*==============================================================================*/
DT_INLINE void _TP_BusOutRestartCondition(void)
{
	portSetCLK(0);
	portSetDATA(1);
	portSetCLK(1);
	portSetDATA(0);
}

/*==============================================================================*/
/*  Func:   _TP_Bus1BitOutDrv                                                   */
/*  Desc:   1bit Data Output Function                                           */
/*==============================================================================*/
DT_INLINE void _TP_Bus1BitOutDrv( DT_UINT dat )
{
	portSetCLK(0);
	portSetDATA( dat );
	portSetCLK(1);
}

/*==============================================================================*/
/*  Func:   _TP_BusOutByteDrv                                                   */
/*  Desc:   Byte Data Output Function                                           */
/*==============================================================================*/
DT_INLINE void _TP_BusOutByteDrv( DT_UINT dat )
{
	_TP_Bus1BitOutDrv(dat & 0x80);
	_TP_Bus1BitOutDrv(dat & 0x40);
	_TP_Bus1BitOutDrv(dat & 0x20);
	_TP_Bus1BitOutDrv(dat & 0x10);
	_TP_Bus1BitOutDrv(dat & 0x08);
	_TP_Bus1BitOutDrv(dat & 0x04);
	_TP_Bus1BitOutDrv(dat & 0x02);
	_TP_Bus1BitOutDrv(dat & 0x01);
	_TP_Bus1BitOutDrv(0);							/* ACK */
}

/*==============================================================================*/
/*  Func: _TP_BusOutDrv                                                         */
/*  Desc: Test Point Output Function                                            */
/*==============================================================================*/
DT_INLINE void _TP_BusOutDrv( DT_UINT addr, DT_UINT dat )
{
	_TP_BusOutByteDrv( dat >> 8 );
	_TP_BusOutByteDrv( dat );
	_TP_BusOutByteDrv( addr >> 8 );
	_TP_BusOutByteDrv( addr );
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutDrv                                                    */
/*  Desc:   Value Output Function                                               */
/*==============================================================================*/
DT_INLINE void _TP_MemoryOutDrv( DT_UINT addr, DT_UINT dat, unsigned char *p, DT_UINT size )
{
	_TP_BusOutStartCondition();
	_TP_BusOutDrv( addr | DT_VARIABLE_BIT, dat );
	_TP_BusOutRestartCondition();

	/* output value */
	if( size >= 256 ) size = 256;
	_TP_BusOutByteDrv( size );
	for( ; size != 0; --size, ++p ){
		_TP_BusOutByteDrv( *p );
	}

	_TP_BusOutStopCondition();
}

/*==============================================================================*/
/*  Func: _TP_BusOutBuf                                                         */
/*  Desc: Called by Test Point                                                  */
/*==============================================================================*/
void _TP_BusOutBuf( unsigned int dat )
{
	logBitBuf[dat/8] |= 1 << (dat & 7);
}

/*==============================================================================*/
/*  Func: _TP_SaveCoverageData                                                  */
/*  Desc: Save Coverage Data Buffer to file.                                    */
/*        to be called when want to save data                                   */
/*==============================================================================*/
void _TP_SaveCoverageData(void)
{
	int i;
	unsigned char buff[256];
	unsigned int pktn = 0;

	portInit();
	bufPos = 0;
	while (bufPos < MAX_LOG_BIT_SIZE) {
		buff[(pktn*8)+7] = 0;
		for (i = 0; i < 7; i++) {
			if (bufPos < MAX_LOG_BIT_SIZE) {
				buff[(pktn*8)+i] = logBitBuf[bufPos++];
			} else {
				buff[(pktn*8)+i] = 0;
			}
			buff[(pktn*8)+7] += buff[(pktn*8)+i];
		}
		pktn++;
		if ((pktn == 32) || (bufPos >= MAX_LOG_BIT_SIZE)) {
			enterCritical();
			_TP_MemoryOutDrv(0x12, 0x01, buff, (pktn * 8));
			exitCritical();
			pktn = 0;
		}
	}
}

